@@ -5,25 +5,19 @@
 #' the errors (\code{stopifnot()} only prints the possibly truncated source code
 #' of the expressions).
 #'
-#' There are two ways to write R expressions in the \code{...} argument.
-#'
-#' The first way is a single R expression wrapped in \code{{}} and passed as a
-#' single argument. This expression may contain multiple sub-expressions. A
+#' For the \code{...} argument, it should be a single R expression wrapped in
+#' \code{{}}. This expression may contain multiple sub-expressions. A
 #' sub-expression is treated as a test condition if it is wrapped in \code{()}
 #' (meaning its value will be checked to see if it is a logical vector
 #' containing any \code{FALSE} values) , otherwise it is evaluated in the normal
 #' way and its value will not be checked. If the value of the last
 #' sub-expression is logical, it will also be treated as a test condition.
-#'
-#' The second way is a series of R expressions (each expression is passed as an
-#' individual argument) that return vectors of \code{TRUE}'s (if \code{FALSE} is
-#' returned anywhere, an error will show up).
 #' @param fact a message for the assertions when any of them fails; treated the
 #'   same way as expressions in \code{...} if it is not a character string,
 #'   which means you do not have to provide a message to this function
-#' @param ... any number of R expressions; see Details
+#' @param ... an R expression; see Details
 #' @return For \code{assert()}, invisible \code{NULL} if all expressions
-#'   returned \code{TRUE}, otherwise an error is signalled and the user-provided
+#'   returned \code{TRUE}, otherwise an error is signaled and the user-provided
 #'   message is emitted. For \code{\%==\%}, \code{TRUE} or \code{FALSE}.
 #' @note The internal implementation of \code{assert()} is different with the
 #'   \code{stopifnot()} function in R \pkg{base}: (1) the custom message
@@ -145,12 +139,14 @@ assert2 = function(fact, exprs, envir, all = TRUE) {
 #' when running tests, which allows one to use non-exported objects in the
 #' package without having to resort to the triple colon \code{\link{:::}} trick.
 #'
-#' The tests are assumed to be under the \file{testit/} directory by default,
-#' and this function also looks for the \file{tests/testit/} directory under the
-#' package installation directory when the user-provided \code{dir} does not
-#' exist. The test scripts must be named of the form \samp{test-*.R}; other R
-#' scripts will not be treated as test files (but may also be useful, e.g. you
-#' can \code{\link{source}()} them in tests).
+#' The tests are assumed to be under the \file{testit/} or \file{tests/testit/}
+#' directory by default (depending on your working directory is the package root
+#' directory or the \file{tests/} directory). This function also looks for the
+#' \file{tests/testit/} directory under the package installation directory when
+#' the user-provided \code{dir} does not exist. The test scripts must be named
+#' of the form \samp{test-*.R}; other R scripts will not be treated as test
+#' files (but may also be useful, e.g. you can \code{\link{source}()} them in
+#' tests).
 #'
 #' For \command{R CMD check}, this means the test R scripts (\file{test-*.R} are
 #' under \file{pkg_root/tests/testit/}. The R scripts are executed with
@@ -160,14 +156,37 @@ assert2 = function(fact, exprs, envir, all = TRUE) {
 #' be removed before the code is executed.
 #' @param package the package name
 #' @param dir the directory of the test files; by default, it is the directory
-#'   \file{testit/} under the current working directory
+#'   \file{testit/} or \file{tests/testit/} under the current working directory
 #' @return \code{NULL}. All test files are executed, unless an error occurs.
 #' @note All test scripts (\samp{test-*.R}) must be encoded in UTF-8 if they
 #'   contain any multibyte characters.
 #' @seealso The \pkg{testthat} package (much more sophisticated).
 #' @export
 #' @examples \dontrun{test_pkg('testit')}
-test_pkg = function(package, dir = 'testit') {
+test_pkg = function(package, dir = c('testit', 'tests/testit')) {
+  # install the source package before running tests when this function is called
+  # in a non-interactive R session that is not `R CMD check`
+  install = !.env$installed && !interactive() &&
+    file.exists(desc <- file.path('../DESCRIPTION')) &&
+    is.na(Sys.getenv('_R_CHECK_PACKAGE_NAME_', NA)) &&
+    !is.na(p <- read.dcf(desc, fields = 'Package')[1, 1]) && p == package
+  if (install) {
+    .env$lib_old = lib_old = .libPaths()
+    .env$lib_new = lib_new = tempfile('R-lib-', '.'); dir.create(lib_new)
+    res = system2(
+      file.path(R.home('bin'), 'R'), c(
+        'CMD', 'INSTALL', paste0('--library=', lib_new),
+        '--no-help', '--no-staged-install', '--no-test-load', '..'
+      )
+    )
+    if (res == 0) {
+      .libPaths(c(lib_new, lib_old))
+      .env$installed = TRUE
+    }
+  }
+  if (!is.na(i <- match(paste0('package:', package), search())))
+    detach(pos = i, unload = TRUE, force = TRUE)
+
   library(package, character.only = TRUE)
   path = available_dir(c(dir, system.file('tests', 'testit', package = package)))
   rs = list.files(path, '^test-.+[.][rR]$', full.names = TRUE)
